/***
*		DurFilt.c -- duration filter for b&p
*		 1996,1997 Michael Rees (aka Another Option)   All rights reserved.
***/

/*
=============================================================================
|   DATE   | WHAT
=============================================================================
 22 Nov 96 | created
 30 Nov 96 | added </>
  2 Sep 97 | sprintf buffer too small in hiloCCName, asshole
  3 Sep 97 | fixed menu for cc number/name display in toolpad
*/


#define DBUG 0

/* define this to write to sd0:BP.dump instead of con: for dprintf */
//#define FILEDUMP

#include <exec/types.h>
#include <exec/memory.h>
#include <dos/dos.h>
#include <intuition/intuition.h>
#include <string.h>

#include <proto/exec.h>
#include <proto/intuition.h>
#include <proto/graphics.h>
#include <clib/alib_protos.h>
#include <clib/alib_stdio_protos.h>
//#include <stdio.h>
#include <stdarg.h>

#include "bars.h"
#include "BarsMacros.h"
#include "BPLib.h"
//#include "configlib.h"
#include "PopUpPipes.h"
#include "durfiltw.h"
#include "snapshotgadget.h"
#include "proto/durfilt_protos.h"
#include "ToolID.h"
#include "debug.h"
#include "/icons/durationfilter.h"

static UBYTE const versiontag[] = "$VER: DurationFilter 40.2 (03.09.97)";

#define SX_LENGTH 14
/* range for duration pots */
#define MAXDUR 1536

struct DurTool {
	struct Tool tool;
	UWORD flags;
	UWORD duration; // value to test
	UWORD mindur;   // range for cc mapping
	UWORD maxdur;
	UBYTE durcc;    // cc to map to duration
	UBYTE oncc;     // ?cc to turn tool on & off?
	UWORD durnow;   // current cc mapped duration
	UWORD res;      // quantize value for mapped durations
	UWORD _word_pad;
};

#define TOOL_ON          1
#define USE_CCMAP        2
#define USE_CCON         4
#define BRANCH_ONCC      8
#define DISCARD_ONCC  0X10
#define BRANCH_DURCC  0x20
#define DISCARD_DURCC 0X40
#define GREATER       0x80
#define USE_SYSEX    0x100
#define FREE_SYSEX   0x200
#define SHOW_CCNUM   0x400

#define TOOL_SIZE sizeof(struct DurTool)
#define TOOLID DrFi_TOOLID
#define SECRET_MESSAGE 9999

extern struct IntuitionBase *IntuitionBase;
extern struct GfxBase *GfxBase;
extern struct LayersBase *LayersBase;
extern struct DOSBase *DOSBase;
extern struct ExecBase *SysBase;
extern struct Functions *functions;

/*==========================================================================
* inittool
*
*  INPUT: pointer to tool
* RETURN: none
*
* EFFECT: sets tool defaults
*=========================================================================*/

static void __regargs inittool(struct DurTool *tool)

{
	tool->flags = DISCARD_DURCC | DISCARD_ONCC | FREE_SYSEX | TOOL_ON;
	tool->duration = 96;
	tool->mindur = tool->durnow = 48;
	tool->maxdur = 192;
	tool->durcc = 13;
	tool->oncc = 113;
	tool->res = 24;
	tool->tool.touched |= TOUCH_INIT;
}

/****************************/
#define REFRESH_BOOLEAN   1
#define REFRESH_DUR       2
#define REFRESH_POTS      4
#define REFRESH_POPPIPES  8
#define REFRESH_CCPOTS 0x10
#define REFRESH_POPUPS 0x20

struct RefreshMsg {
	struct Message message;
	UBYTE bits;
};

static void __regargs SendSecretMessage(struct DurTool *tool,UBYTE bits)
{
	struct RefreshMsg *msg;

	if (msg=(struct RefreshMsg *)functions->myalloc(sizeof(struct RefreshMsg),0)) {
		msg->message.mn_Node.ln_Type = NT_MESSAGE;
		msg->message.mn_ReplyPort = NULL;
		msg->message.mn_Length = SECRET_MESSAGE;
		msg->bits = bits;
		PutMsg(tool->tool.window->UserPort,(struct Message *)msg);
	}
}

/*==========================================================================
* dosysex
*
*  INPUT: pointer to StringEvent
* RETURN: pointer to Event
*
* EFFECT: gets new tool state from sysex message
*=========================================================================*/

static __inline struct Event *dosysex(struct DurTool *tool,
                                      struct StringEvent *event)

{
	register unsigned char *sxdata;
	UWORD sxflag;

	if (tool->flags & USE_SYSEX) {
		if (sxdata = checksysex(event,TOOL_SYSEX_ID,tool->tool.toolid)) {
			sxflag = tool->flags & (USE_SYSEX | FREE_SYSEX);
			memcpy((void *)&tool->flags,(void *)sxdata,SX_LENGTH);
			if (sxflag & USE_SYSEX) tool->flags |= USE_SYSEX;
			if (sxflag & FREE_SYSEX) tool->flags |= FREE_SYSEX;
			if (tool->tool.window) SendSecretMessage(tool,0xFF);
			if (tool->flags & FREE_SYSEX) {
				functions->freestring(event->string);
				functions->freeevent((struct Event *)event);
				event=NULL;
			}
			else event->tool= tool->tool.next;
		}
	}
	return((struct Event *)event);
}

/* coordinates for display of current cc mapped duration */
#define DURNOW_X 222
#define DURNOW_Y  60

/**** display the current cc mapped duration */
static void __regargs showdur(struct DurTool *tool)
{
	char text[13];

	functions->lengthtostring(tool->tool.clip,0,
	               (tool->flags & USE_CCMAP) ? tool->durnow : tool->duration,text);
	SetRPort(tool->tool.window->RPort, DURNOW_X, DURNOW_Y, 2, 0, JAM2);
	Text(tool->tool.window->RPort,&text[3],8);
}

#define BITE_ME 0x0F

/*** doevent() *****************************/
static void  __regargs doevent(struct DurTool *tool, struct NoteEvent *event)

{
	UWORD dur;

	event->tool = tool->tool.next;
	if (IsNoteOn(event)) {
		if (tool->flags & TOOL_ON) {
			if (tool->flags & USE_CCMAP) dur = tool->durnow;
			else dur = tool->duration;
			if ( ((!(tool->flags & GREATER)) && (event->duration < dur)) ||
			     ((tool->flags & GREATER) && (event->duration > dur)) ) {
				if (tool->tool.branch) {
					event->tool = tool->tool.branch;
					event->type |= EVENT_BRANCH;
				}
				else event->type = BITE_ME;
			}
		}
	}
	else if (IsCtrlChg((struct Event *)event)) {
		dur = 0; // repository for refresh bits
		if ((tool->flags & USE_CCON) && (event->value == tool->oncc)) {
			dur = REFRESH_BOOLEAN;
			if (SwitchOn(event->velocity)) tool->flags |= TOOL_ON;
			else tool->flags &= ~TOOL_ON;
			if (tool->flags & BRANCH_ONCC) {
				if (tool->tool.branch) {
					event->tool = tool->tool.branch;
					event->type |= EVENT_BRANCH;
				}
				else event->type = BITE_ME;
			}
			else if (tool->flags & DISCARD_ONCC) event->type = BITE_ME;
		}
		if ((tool->flags & USE_CCMAP) && (event->value == tool->durcc)) {
			dur |= REFRESH_DUR;
#pragma msg 100 ignore push
			tool->durnow = 
				IMulU(((tool->maxdur-tool->mindur)/128),event->velocity)+tool->mindur;
			if (tool->res) { //quantize that puppy
				tool->durnow += (tool->res >> 1);
				tool->durnow = IDivU(tool->durnow,tool->res);
				tool->durnow = IMulU(tool->durnow,tool->res);
#pragma msg 100 pop
				if (tool->durnow < 1) tool->durnow = tool->res;
			}
			if (tool->flags & BRANCH_DURCC) {
				if (tool->tool.branch) {
					event->tool = tool->tool.branch;
					event->type |= EVENT_BRANCH;
				}
				else event->type = BITE_ME;
			}
			else if (tool->flags & DISCARD_DURCC) event->type = BITE_ME;
		}
		if (dur && tool->tool.window) SendSecretMessage(tool,(UBYTE)dur);
	}
}

/*==========================================================================
* processlist
*
*  INPUT: pointers to tool and list to process, start and stop times
* RETURN: pointer to inverted list
*
* EFFECT: processes a list of events
*=========================================================================*/

static struct Event * __saveds __stdargs processlist(struct DurTool *tool,
                                                     struct NoteEvent *list,
                                                     long start, long stop)
{
	struct NoteEvent *event = list;

	if (!tool->tool.touched) inittool(tool);
	if (start < 0) start = 0;
	while (event->time < start) event = event->next;
	while (event && (event->time <= stop)) {
		doevent(tool,event);
		if (event->type == BITE_ME) event->time = -1;
		event = event->next;
	}
	list = (struct NoteEvent *)functions->sorteventlist((struct Event *)list);
	for ( event=list ; event ; event=list ) {
		if (event->time >= 0) break;
		else {
			list = event->next;
			functions->freeevent((struct Event *)event);
		}
	}
	return((struct Event *)list);
}

/*==========================================================================
* processevent
*
*  INPUT: pointer to Event
* RETURN: pointer to Event
*
* EFFECT: processes events one at a time
*=========================================================================*/

static struct Event * __saveds __stdargs processevent(struct NoteEvent *event)

{
   struct DurTool *tool = (struct DurTool *)event->tool;

	if (IsSysEx((struct Event *)event)) {
		event = (struct NoteEvent *)dosysex(tool,(struct StringEvent *)event);
	}
	else if (IsNoteOff(event)) {
		functions->freeevent((struct Event *)event);
		event = NULL;
	}
	else {
		doevent(tool,event);
		if (event->type == BITE_ME) {
			functions->freeevent((struct Event *)event);
			event = NULL;
		}
		else if (IsNoteOn(event)) {
			event->next = CloneNoteOff(event);
			if (!event->next) {
				functions->freeevent((struct Event *)event);
				event = NULL;
			}
		}
	}
	return((struct Event *)event);
}

static struct DurTool *selectedtool;

/***
 *** displays time m.bb.ccc, with a minimum of 1, for prop gadget 
 ***/

static long __saveds __stdargs hiloShort(struct Window *w, struct Gadget *g, long  x)
{
	char text[13];

	if (x==0) x=1;
	functions->lengthtostring(selectedtool->tool.clip,0,x,text);
	SetRPort(w->RPort, g->LeftEdge+g->Width+XOffset, g->TopEdge+YOffset,
            (g->Flags & GADGDISABLED)?HILO_DPEN:HILO_APEN,
				HILO_BPEN, HILO_DRAWMODE);
	Text(w->RPort,&text[3],8);
	return((long)x);
}

/***
 *** displays time m.bb.ccc, with 0==Off, for prop gadget 
 ***/

static long __saveds __stdargs hiloShort0(struct Window *w, struct Gadget *g, long  x)
{
	char text[13];

	if (x) functions->lengthtostring(selectedtool->tool.clip,0,x,text);
	else strcpy(&text[3],"Off     ");
	SetRPort(w->RPort, g->LeftEdge+g->Width+XOffset, g->TopEdge+YOffset,
            (g->Flags & GADGDISABLED)?HILO_DPEN:HILO_APEN,
				HILO_BPEN, HILO_DRAWMODE);
	Text(w->RPort,&text[3],8);
	return((long)x);
}

/*==========================================================================
* hiloCCName
*
*  INPUT: pointers to window and gadget, gadget value
* RETURN: gadget value
*
* EFFECT: display a pot value mapped to controller name
*=========================================================================*/

static long __stdargs __saveds hiloCCName(struct Window *w,struct Gadget *g,long x)

{
	static char *name[] = {
/*0 - 4 */	"MonPres", "ModWhel", "Breath ", "MIDI 03", "Foot   ",
/*5 - 9 */	"PortTim", "DataEnt", "Volume ", "Balance", "MIDI 09",
/*10-14 */	"Pan    ", "Express", "MIDI 12", "MIDI 13", "MIDI 14",
/*15-19 */	"MIDI 15", "Genl 1 ", "Genl 2 ", "Genl 3 ", "Genl 4 ",
/*20-24 */	"MIDI 20", "MIDI 21", "MIDI 22", "MIDI 23", "MIDI 24",
/*25-29 */	"MIDI 25", "MIDI 26", "MIDI 27", "MIDI 28", "MIDI 29",
/*30-34 */	"MIDI 30", "MIDI 31", "MIDI 32", "MIDI 33", "MIDI 34",
/*35-39 */	"MIDI 35", "MIDI 36", "MIDI 37", "MIDI 38", "MIDI 39",
/*40-44 */	"MIDI 40", "MIDI 41", "MIDI 42", "MIDI 43", "MIDI 44",
/*45-49 */	"MIDI 45", "MIDI 46", "MIDI 47", "MIDI 48", "MIDI 49",
/*50-54 */	"MIDI 50", "MIDI 51", "MIDI 52", "MIDI 53", "MIDI 54",
/*55-59 */	"MIDI 55", "MIDI 56", "MIDI 57", "MIDI 58", "MIDI 59",
/*60-64 */	"MIDI 60", "MIDI 61", "MIDI 62", "MIDI 63", "Sustain",
/*65-69 */	"PortSwt", "Sosten.", "SoftPed", "MIDI 68", "FrezPdl",
/*70-74 */	"MIDI 70", "MIDI 71", "MIDI 72", "MIDI 73", "MIDI 74",
/*75-79 */	"LegtoSw", "MIDI 76", "MIDI 77", "MIDI 78", "MIDI 79",
/*80-84 */	"Genl 5 ", "Genl 6 ", "Genl 7 ", "Genl 8 ", "MIDI 84",
/*85-89 */	"MIDI 85", "MIDI 86", "MIDI 87", "MIDI 88", "MIDI 89",
/*90-94 */	"MIDI 90", "FXDepth", "TremDep", "ChorDep", "CelDep ",
/*95-99 */	"PhsrDep", "DataInc", "DataDec", "MIDI 98", "MIDI 99",
/*100-104*/	"MIDI100", "MIDI101", "MIDI102", "MIDI103", "MIDI104",
/*105-109*/	"MIDI105", "MIDI106", "MIDI107", "MIDI108", "MIDI109",
/*110-114*/	"MIDI110", "MIDI111", "MIDI112", "MIDI113", "MIDI114",
/*115-119*/	"MIDI115", "MIDI116", "MIDI117", "MIDI118", "MIDI119",
/*120-124*/	"MIDI120", "RsetCtl" /*, "LoclCtl", "AllOff ", "OmniOff",*/
/*125-127*/	/*"Omni On", "Mono On", "Poly On"*/
	};

	char text[8];

	SetRPort(w->RPort, g->LeftEdge+g->Width+XOffset, g->TopEdge+YOffset,
	         (g->Flags & GADGDISABLED) ? BP_DGREY : BP_WHITE, 0, JAM2);
	if (selectedtool->flags & SHOW_CCNUM) {
		sprintf(text,"%3ld",x);
		strcat(&text[3],"    ");
		Text(w->RPort,text,7);
	}
	else {
		Text(w->RPort,name[x],7);
	}
	return(x);
}

/*==========================================================================
* dosnapshot
*
*  INPUT: pointer to tool
* RETURN: none
*
* EFFECT: makes a sysex message and posts it
*=========================================================================*/

static void __regargs dosnapshot(struct DurTool *tool)

{
	UBYTE sx[SX_LENGTH];
	struct StringEvent *event;
	
	memcpy(sx,&tool->flags,SX_LENGTH);
	if (event = BuildSnapshot(&tool->tool, 
	                          functions->running ? functions->timenow : -1,
	                          sx, SX_LENGTH)) {
		PostSnapshot(&tool->tool,event);
	}
}

#define tw tool->tool.window
#ifndef GDOWN
 #define GDOWN 100
 #define GUP 200
#endif
#define pipemenu &PopPipeMenu
/*==========================================================================
* edittool
*
*  INPUT: pointer to tool
* RETURN: none
*
* EFFECT: ui handler
*=========================================================================*/

static void __saveds __stdargs edittool(struct DurTool *tool)

{
	struct IntuiMessage *message;
	long class; 
	long code;
	struct Gadget *gadget;
	struct NewWindow *nw;
	struct Menu *menu;
	struct MenuItem *menuitem;
	UBYTE refresh;

	if (!(tool->tool.touched & TOUCH_INIT)) inittool(tool);
	if (InToolPad((struct Tool *)tool) || !tool->tool.track) refresh = 0;
	else refresh = 20;
	nw=OpenEmbossedWindowSlop(&NewWindowStructure1,&tool->tool,0,WTITLE,refresh);
	if (!nw) return;
	if (tool->flags & SHOW_CCNUM) MenuItem9.Flags |= CHECKED;
	else MenuItem9.Flags &= ~CHECKED;
	if (refresh) {
		if (tool->flags & USE_SYSEX) MenuItem10.Flags |= CHECKED;
		else MenuItem10.Flags &= ~CHECKED;
		if (tool->flags & FREE_SYSEX) MenuItem11.Flags |= CHECKED;
		else MenuItem11.Flags &= ~CHECKED;
		AddSnapshotGadget(tw);
	}
	else { // no sysex in toolpad
		MenuItem9.NextItem = NULL;
	}
	if (menu = functions->DupeMenu(&MenuList1)) SetMenuStrip(tw,menu);
	else SetWindowTitles(tw,(UBYTE *)-1,MenuList1.MenuName);
	DrawBevelBox(tw->RPort,BORDER_X1,BORDER_Y1,BORDER_X2,BORDER_Y2,-1,-1,1);
	PrintIText(tw->RPort,&IntuiTextList1,0,0);
	EmbossOnGList(tw,&GadgetList1,GCOUNT,1);
	functions->FatEmbossedPropOn(tw,DUR,DUR+GDOWN,DUR+GUP,hiloShort,MAXDUR+1,1);
	functions->FatEmbossedPropOn(tw,MAX,MAX+GDOWN,MAX+GUP,hiloShort,MAXDUR+1,1);
	functions->FatEmbossedPropOn(tw,MIN,MIN+GDOWN,MIN+GUP,hiloShort,MAXDUR+1,1);
	functions->FatEmbossedPropOn(tw,RES,RES+GDOWN,RES+GUP,hiloShort0,769,1);
	functions->FatEmbossedPropOn(tw,CC,CC+GDOWN,CC+GUP,hiloCCName,121,1);
	functions->FatEmbossedPropOn(tw,ONOFFCC,ONOFFCC+GDOWN,ONOFFCC+GUP,hiloCCName,121,1);
	functions->RefreshGadget(tw,POPUP_DUR);
	functions->RefreshGadget(tw,POPUP_MAX);
	functions->RefreshGadget(tw,POPUP_MIN);
	refresh = 255;

	/*** ui handling ***/
	for ( ; ; ) {
		if (refresh) {
			if (refresh & REFRESH_BOOLEAN) {
				functions->SelectEmbossed(tw,CCMAP,tool->flags & USE_CCMAP);
				functions->SelectEmbossed(tw,CC_SWITCH,tool->flags & USE_CCON);
				functions->SelectEmbossed(tw,ON,tool->flags & TOOL_ON);
				functions->SelectEmbossed(tw,OFF,!(tool->flags & TOOL_ON));
				functions->SelectEmbossed(tw,GT,tool->flags & GREATER);
#pragma msg 104 ignore push
				SetGadgetText(tw,GT,(tool->flags & GREATER) ? ">" : "<");
#pragma msg 104 pop
				DrawEmbossedGList(tw,functions->GetGadget(tw,CC_SWITCH),GCOUNT);
			}
			if (refresh & REFRESH_POPPIPES) {
				gadget = functions->GetGadget(tw,POP_ON);
				if (tool->flags & BRANCH_ONCC) gadget->GadgetRender = (APTR)&ElbowImage;
				else if (tool->flags & DISCARD_ONCC) gadget->GadgetRender = (APTR)&PlugImage;
				else gadget->GadgetRender = (APTR)&PipeImage;
				functions->RefreshGadget(tw,POP_ON);
				gadget = functions->GetGadget(tw,POP_CC);
				if (tool->flags & BRANCH_DURCC) gadget->GadgetRender = (APTR)&ElbowImage;
				else if (tool->flags & DISCARD_DURCC) gadget->GadgetRender = (APTR)&PlugImage;
				else gadget->GadgetRender = (APTR)&PipeImage;
				functions->RefreshGadget(tw,POP_CC);
			}
			if (refresh & REFRESH_POPUPS) {
				functions->EnableGadget(tw,POPUP_DUR,!(tool->flags & USE_CCMAP));
				functions->EnableGadget(tw,POPUP_MAX,tool->flags & USE_CCMAP);
				functions->EnableGadget(tw,POPUP_MIN,tool->flags & USE_CCMAP);
				functions->EnableGadget(tw,POPUP_RES,tool->flags & USE_CCMAP);
				for (code=POPUP_DUR ; code <= POPUP_RES ; code++) 
					functions->RefreshGadget(tw,code);
			}
			selectedtool = tool;
			if (refresh & REFRESH_POTS) {
				if (tool->flags & USE_CCMAP) { // do some sanity checking
					if (tool->res) {
						if (abs(tool->maxdur-tool->mindur) < tool->res) {
							tool->res = abs(tool->maxdur-tool->mindur);
							if (tool->res < 1) tool->res = 1;
						}
						code = tool->res;
					}
					else code = 1;
					if (tool->maxdur < tool->mindur) tool->maxdur = tool->mindur+code;
					if (tool->maxdur > MAXDUR) {
						tool->maxdur = MAXDUR;
						tool->mindur = MAXDUR - code;
					}
					if (tool->mindur >= tool->maxdur) tool->mindur = tool->maxdur-code;
					if (tool->mindur < 1) {
						tool->mindur = 1;
						tool->maxdur = tool->mindur + code;
					}
					refresh |= REFRESH_DUR;
				}
				EnableFatProp(tw, MAX,MAX+GDOWN,MAX+GUP,tool->flags & USE_CCMAP);
				functions->ModifyEmbossedProp(tw,MAX,tool->maxdur,0,0,0,0,0);
				functions->DrawEmbossedProp(tw,MAX);
				EnableFatProp(tw, MIN,MIN+GDOWN,MIN+GUP,tool->flags & USE_CCMAP);
				functions->ModifyEmbossedProp(tw,MIN,tool->mindur,0,0,0,0,0);
				functions->DrawEmbossedProp(tw,MIN);
				EnableFatProp(tw, RES,RES+GDOWN,RES+GUP,tool->flags & USE_CCMAP);
				functions->ModifyEmbossedProp(tw,RES,tool->res,0,0,0,0,0);
				functions->DrawEmbossedProp(tw,RES);
				functions->EnableGadget(tw,POP_CC,tool->flags & USE_CCMAP);
				functions->EnableGadget(tw,POP_ON,tool->flags & USE_CCON);
			}
			if (refresh & REFRESH_CCPOTS) {
				EnableFatProp(tw,CC,CC+GDOWN,CC+GUP,tool->flags & USE_CCMAP);
				functions->ModifyEmbossedProp(tw,CC,tool->durcc,0,0,0,0,0);
				functions->DrawEmbossedProp(tw,CC);
				EnableFatProp(tw,ONOFFCC,ONOFFCC+GDOWN,ONOFFCC+GUP,tool->flags & USE_CCON);
				functions->ModifyEmbossedProp(tw,ONOFFCC,tool->oncc,0,0,0,0,0);
				functions->DrawEmbossedProp(tw,ONOFFCC);
			}
			if (refresh & REFRESH_DUR) {
				EnableFatProp(tw, DUR,DUR+GDOWN,DUR+GUP,!(tool->flags & USE_CCMAP));
				functions->ModifyEmbossedProp(tw,DUR,tool->duration,0,0,0,0,0);
				functions->DrawEmbossedProp(tw,DUR);
				showdur(tool);
			}
			refresh = 0;
		}
		message = (struct IntuiMessage *)functions->GetIntuiMessage(tw);
		if (message->ExecMessage.mn_Length == SECRET_MESSAGE) {
			refresh = ((struct RefreshMsg *)message)->bits;
			functions->myfree((char *)message,sizeof(struct RefreshMsg));
			continue;
		}
		/* else */
		class = message->Class;
		code = message->Code;
		gadget = (struct Gadget *)message->IAddress;
		ReplyMsg((struct Message *)message);
		class = functions->SystemGadgets(tw,class,gadget,code);

		if (class == CLOSEWINDOW) {
			break;
		}

		else if (class == MENUPICK) {
			while (code != MENUNULL) {
				menuitem = ItemAddress(menu,code);
				switch (ITEMNUM(code)) {
					case 0:
						tool->flags ^= SHOW_CCNUM;
						refresh = REFRESH_CCPOTS;
						break;
					case 1:
						tool->flags ^= USE_SYSEX;
						break;
					case 2:
						tool->flags ^= FREE_SYSEX;
						break;
					case 3:
						dosnapshot(tool);
						break;
				}
				code = menuitem->NextSelect;
			}
		}

		else if (class == GADGETUP) {
			switch (gadget->GadgetID) {
				case SNAPSHOT_GADGET_ID :
					dosnapshot(tool);
					break;
				case CCMAP :
					tool->flags ^= USE_CCMAP;
					refresh = REFRESH_BOOLEAN|REFRESH_POTS|REFRESH_CCPOTS|REFRESH_DUR|REFRESH_POPUPS;
					break;
				case ON :
					tool->flags ^= TOOL_ON;
					refresh = REFRESH_BOOLEAN;
					break;
				case OFF :
					tool->flags ^= TOOL_ON;
					refresh = REFRESH_BOOLEAN;
					break;
				case CC_SWITCH :
					tool->flags ^= USE_CCON;
					refresh = REFRESH_BOOLEAN|REFRESH_POPPIPES|REFRESH_POTS|REFRESH_CCPOTS|REFRESH_POPUPS;
					break;
				case GT :
					tool->flags ^= GREATER;
					refresh = REFRESH_BOOLEAN;
					break;
			}
		}

		else if (class == GADGETDOWN) {
			selectedtool = tool;
			switch (gadget->GadgetID) {
				case POPUP_DUR :
					code = functions->popupnote(tw,-1);
					if (code >= 0) {
						tool->duration = 768 >> code;
					}
					refresh = REFRESH_DUR;
					break;
				case POPUP_MIN :
					code = functions->popupnote(tw,-1);
					if (code >= 0) {
						tool->mindur = 768 >> code;
					}
					refresh = REFRESH_POTS;
					break;
				case POPUP_MAX :
					code = functions->popupnote(tw,-1);
					if (code >= 0) {
						tool->maxdur = 768 >> code;
					}
					refresh = REFRESH_POTS;
					break;
				case POPUP_RES :
					code = functions->popupnote(tw,-1);
					if (code >= 0) {
						tool->res = 768 >> code;
					}
					refresh = REFRESH_POTS;
					break;

				case POP_CC :
					code = (short)functions->PopUpMenu(tw,pipemenu);
					if (ITEMNUM(code) < 3) {
						switch (ITEMNUM(code)) {
							case PIPE_ITEM : // pipe selected
								tool->flags &= ~(DISCARD_DURCC|BRANCH_DURCC);
								break;
							case PLUG_ITEM : // plug selected
								tool->flags = (tool->flags & ~BRANCH_DURCC)|DISCARD_DURCC;
								break;
							case ELBOW_ITEM : // elbow selected
								tool->flags = (tool->flags & ~DISCARD_DURCC)|BRANCH_DURCC;
								break;
						}
						refresh = REFRESH_POPPIPES;
					}
					break;
				case POP_ON :
					code = (short)functions->PopUpMenu(tw,pipemenu);
					if (ITEMNUM(code) < 3) {
						switch (ITEMNUM(code)) {
							case PIPE_ITEM : // pipe selected
								tool->flags &= ~(DISCARD_ONCC|BRANCH_ONCC);
								break;
							case PLUG_ITEM : // plug selected
								tool->flags = (tool->flags & ~BRANCH_ONCC)|DISCARD_ONCC;
								break;
							case ELBOW_ITEM : // elbow selected
								tool->flags = (tool->flags & ~DISCARD_ONCC)|BRANCH_ONCC;
								break;
						}
						refresh = REFRESH_POPPIPES;
					}
					break;

				case DUR :
					tool->duration = functions->DragEmbossedProp(tw,DUR);
					refresh = REFRESH_DUR;
					break;
				case DUR+GUP :
					tool->duration = functions->ShiftEmbossedProp(tw,DUR,1,0);
					refresh = REFRESH_DUR;
					break;
				case DUR+GDOWN :
					tool->duration = functions->ShiftEmbossedProp(tw,DUR,-1,0);
					refresh = REFRESH_DUR;
					break;

				case MAX :
					tool->maxdur = functions->DragEmbossedProp(tw,MAX);
					refresh = REFRESH_POTS|REFRESH_DUR;
					break;
				case MAX+GUP :
					tool->maxdur = functions->ShiftEmbossedProp(tw,MAX,1,0);
					refresh = REFRESH_POTS|REFRESH_DUR;
					break;
				case MAX+GDOWN :
					tool->maxdur = functions->ShiftEmbossedProp(tw,MAX,-1,0);
					refresh = REFRESH_POTS|REFRESH_DUR;
					break;

				case MIN :
					tool->mindur = functions->DragEmbossedProp(tw,MIN);
					refresh = REFRESH_POTS|REFRESH_DUR;
					break;
				case MIN+GUP :
					tool->mindur = functions->ShiftEmbossedProp(tw,MIN,1,0);
					refresh = REFRESH_POTS|REFRESH_DUR;
					break;
				case MIN+GDOWN :
					tool->mindur = functions->ShiftEmbossedProp(tw,MIN,-1,0);
					refresh = REFRESH_POTS|REFRESH_DUR;
					break;

				case RES :
					tool->res = functions->DragEmbossedProp(tw,RES);
					refresh = REFRESH_POTS;
					break;
				case RES+GUP :
					tool->res = functions->ShiftEmbossedProp(tw,RES,1,0);
					refresh = REFRESH_POTS;
					break;
				case RES+GDOWN :
					tool->res = functions->ShiftEmbossedProp(tw,RES,-1,0);
					refresh = REFRESH_POTS;
					break;

				case CC :
					tool->durcc = functions->DragEmbossedProp(tw,CC);
					refresh = REFRESH_CCPOTS;
					break;
				case CC+GUP :
					tool->durcc = functions->ShiftEmbossedProp(tw,CC,1,0);
					refresh = REFRESH_CCPOTS;
					break;
				case CC+GDOWN :
					tool->durcc = functions->ShiftEmbossedProp(tw,CC,-1,0);
					refresh = REFRESH_CCPOTS;
					break;

				case ONOFFCC :
					tool->oncc = functions->DragEmbossedProp(tw,ONOFFCC);
					refresh = REFRESH_CCPOTS;
					break;
				case ONOFFCC+GUP :
					tool->oncc = functions->ShiftEmbossedProp(tw,ONOFFCC,1,0);
					refresh = REFRESH_CCPOTS;
					break;
				case ONOFFCC+GDOWN :
					tool->oncc = functions->ShiftEmbossedProp(tw,ONOFFCC,-1,0);
					refresh = REFRESH_CCPOTS;
					break;
			}
		}
	}
	selectedtool = NULL;
	functions->EmbossOff(tw,CCMAP);
	functions->FatEmbossedPropOff(tw,DUR,DUR+GDOWN,DUR+GUP);
	functions->FatEmbossedPropOff(tw,MAX,MAX+GDOWN,MAX+GUP);
	functions->FatEmbossedPropOff(tw,MIN,MIN+GDOWN,MIN+GUP);
	functions->FatEmbossedPropOff(tw,RES,RES+GDOWN,RES+GUP);
	functions->FatEmbossedPropOff(tw,CC,CC+GDOWN,CC+GUP);
	functions->FatEmbossedPropOff(tw,ONOFFCC,ONOFFCC+GDOWN,ONOFFCC+GUP);
	if (menu) functions->DeleteMenu(menu);
	NewCloseEmbossedWindow(&tool->tool,nw);
}

static struct ToolMaster master;

/*==========================================================================
* inittoolmaster
*
*  INPUT: none
* RETURN: pointer to ToolMaster
*
* EFFECT: initializes.
*=========================================================================*/

struct ToolMaster * __saveds __stdargs inittoolmaster(void)

{
	memset((char *)&master,0,sizeof(struct ToolMaster));
	master.toolsize = TOOL_SIZE;
	master.toolid = TOOLID;
	master.edittool = edittool;
	master.processevent = processevent;
	master.processlist = processlist;
	master.image = &IconDown;
	master.upimage = &IconUp;
	strcpy(master.name,WTITLE);
	master.tooltype = TOOL_NORMAL | TOOL_BRANCHOUT;
	return(&master);
}
